//
// Copyright 2007 Johannes Hofmann <Johannes.Hofmann@gmx.de>
//
// This software may be used and distributed according to the terms
// of the GNU General Public License, incorporated herein by reference.


#ifndef FLTKOFFSCREEN_HH
#define FLTKOFFSCREEN_HH

#include <fltk/Rectangle.h>
#include <fltk/x.h>

class FltkOffscreen
{
	private:
		int w;
		int h;
		unsigned long pixmap;
		GC gc;

	public:
		FltkOffscreen(int w, int h) {
			this->w = w;
			this->h = h;
			pixmap = XCreatePixmap(fltk::xdisplay, fltk::xwindow,
				w, h, fltk::xvisual->depth);
			gc = XCreateGC(fltk::xdisplay, pixmap, 0, NULL);
			XSetForeground(fltk::xdisplay, gc,
				WhitePixel(fltk::xdisplay, fltk::xscreen));
			clear();
		};

		~FltkOffscreen() {
			if (pixmap) XFreePixmap(fltk::xdisplay, pixmap);
			if (gc) XFreeGC(fltk::xdisplay, gc);
		};

		void clear() {
			XFillRectangle(fltk::xdisplay, pixmap, gc, 0, 0, w, h);
		};

		void draw(const fltk::Rectangle &area, int srcx, int srcy) {
			XCopyArea(fltk::xdisplay, pixmap, fltk::xwindow,
				fltk::gc, srcx, srcy,
				area.w(), area.h(), area.x(), area.y());
		};

		unsigned long getPixmap() { return pixmap; };
};

#endif
