//
// Copyright 2007 Johannes Hofmann <Johannes.Hofmann@gmx.de>
//
// This software may be used and distributed according to the terms
// of the GNU General Public License, incorporated herein by reference.

#ifndef GsWidget_H
#define GsWidget_H

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <stdlib.h>

#include <fltk/Window.h>
#include <fltk/Cursor.h>
#include <fltk/x.h>

using namespace fltk;

#include "PostscriptDSC.H"
#include "FltkOffscreen.H"

class GsWidget : public Widget {
	private:
		PostscriptDSC *dsc;
		int gs_win;
		Atom atoms[5];
		pid_t gs_pid;
		int display_x, display_y;
		int in_fd, reload_needed;
		int zoom_percent;
		int initial_width, initial_height;
		Rectangle abs_rect;
		int feeding;
		fltk::Cursor *cursor;
	
		void resize();
		void clear();

	protected:
		int page;
		FltkOffscreen *offscreen;
		int xdpi, ydpi;
		int paper_x, paper_y;

		void draw();
		void get_window_absolute_rect(Rectangle *rect);

	public:
		GsWidget(int X,int Y,int W, int H);

		~GsWidget();

		int open_file(char *f);

		int open_file(int fd);

		int load();

		int load_page(int p);

		int reload();

		int next();

		int prev();

		int handle(int event);

		int handleX11(int ev);

		int zoom(int p);

		int get_page();

		int get_pages();

	private:
		void setProps();

		void exec_gs();

		void kill_gs();

		int fd_copy(int to, int from, size_t len);

		bool gs_active();

		void init_offscreen();
};
#endif
